from typing import List

import requests
from llama_index import Document
import time

from progress.spinner import Spinner
from social_gpt.ingestion.scraper.topmate_scraper import AbstractTopmateScraper


class LinkedinScraper(AbstractTopmateScraper):

    def __initiate_scraping(self):
        scraping_url = self.base_url + "/scrape-linkedin-posts/"
        data = {"username": self.username}
        response = requests.post(scraping_url, data=data, headers=self.get_authorization_header())
        scraping_id = response.json()["scraping_id"]
        scraping_status = "IN_PROGRESS"
        loader = Spinner("Scraping...")
        while scraping_status == "IN_PROGRESS":
            scraping_status_url = self.base_url + f"/scraping-status/?scraping_id={scraping_id}"
            scraping_status_response = requests.get(scraping_status_url, headers=self.get_authorization_header())
            scraping_status = scraping_status_response.json()["status"]
            loader.next()
            time.sleep(1)
        loader.finish()
        if scraping_status == "FAILED":
            raise Exception("Scraping failed")
        return scraping_id

    def __retrieve_scraped_posts(self, scraping_id):
        remaining = True
        posts_data = []
        initial_url = self.base_url + f'/linkedin-scraped-posts?scraping_id={scraping_id}'
        while remaining:
            data_response = requests.get(initial_url, headers=self.get_authorization_header()).json()
            posts_data += data_response['results']
            remaining = data_response['next'] is not None
            initial_url = data_response['next']
        print(f"Total posts scraped: {len(posts_data)}")
        return list(map(lambda post: Document(text=post["text"]), posts_data))

    def scrape(self) -> List[Document]:
        scraping_id = self.__initiate_scraping()
        return self.__retrieve_scraped_posts(scraping_id)



